#include <stdio.h>
#include <math.h>
#include <assert.h>

float fonction_inconnue(float x,float y);
void recherche_minimum(float x0,float y0);
int trouve_min(float V[],int taille);

int main()
{
  recherche_minimum(1,2);

  return 0;
}

void recherche_minimum(float x0,float y0)
{
    //*****************************************
    //Recherche de minimum de fonction_inconnue
    //*****************************************
    //
    // Algorithme:
    //
    // Initialise (x,y) a (x0,y0)
    //
    // Pre-stocker la table de deplacement pour 8 voisins
    //   C'est a dire: deplacement[k]={ (-dx,+dy)[3] (  0,+dy)[2] (+dx,+dy)[1] }
    //                                { (-dx,  0)[4]              (+dx,  0)[0] }
    //                                { (-dx,-dy)[5] (  0,-dy)[6] (+dx,-dy)[7] }
    //
    // Tant que minimum non atteint
    //
    // Affecter a V0 la valeur de fonction_inconnue(x,y)
    // Pour tous les 8 voisins [k] de (x,y)
    //
    //   Soit (x2,y2) le voisin courant de (x,y)
    //      C'est a dire: x2=x +/- dx
    //                    y2=y +/- dy
    //
    //   Affecter a V[k] la valeur de fonction_inconnue(voisin courant)
    //
    // Fin Pour
    //
    // Cherchez le minimum de V[k] pour k=k_optimal
    //
    // Si V[k_optimal] plus petit que V0
    //   Alors avancer (x,y) dans la direction du voisin k_optimal
    // Fin Si
    //
    //
    // Fin Tant que




    //*****************************************
    //Recherche de minimum de fonction_inconnue
    //*****************************************
    //
    // Algorithme:
    //
    // Initialise (x,y) a (x0,y0)
    float x=x0;
    float y=y0;

    // Prestocker la table de deplacement pour 8 voisins
    //   C'est a dire: deplacement[k]={ (-dx,+dy)[3] (  0,+dy)[2] (+dx,+dy)[1] }
    //                                { (-dx,  0)[4]              (+dx,  0)[0] }
    //                                { (-dx,-dy)[5] (  0,-dy)[6] (+dx,-dy)[7] }
    float dx=1e-5;
    float dy=1e-5;
    float deplacement[8][2]={{dx,0},{dx,dy},{0,dy},{-dx,+dy},{-dx,0},{-dx,-dy},{0,-dy},{+dx,-dy}};

    //
    // Tant que minimum non atteint
    //
    int minimum_atteint=0;
    while(minimum_atteint==0)
    {
        // Affecter a V0 la valeur de fonction_inconnue(x,y)
        float V0=fonction_inconnue(x,y);

        // Pour tous les 8 voisins [k] de (x,y)
        float V[8];
        unsigned int k=0;
        for(k=0;k<8;++k)
        {
            //
            //   Soit (x2,y2) le voisin courant de (x,y)
            //      C'est a dire: x2=x +/- dx
            //                    y2=y +/- dy
            //
            float x2=x+deplacement[k][0];
            float y2=y+deplacement[k][1];
            //   Affecter a V[k] la valeur de fonction_inconnue(voisin courant)
            //

            V[k]=fonction_inconnue(x2,y2);
            // Fin Pour
        }
        //
        // Cherchez le minimum de V[k] pour k=k_optimal
        int k_optimal=trouve_min(V,8);

        //
        // Si V[k_optimal] plus petit que V0
        //   Alors avancer (x,y) dans la direction du voisin k_optimal
        // Sinon
        //   minimum_atteint est vrai
        // Fin Si

        if(V[k_optimal]<V0)
        {
            x += deplacement[k_optimal][0];
            y += deplacement[k_optimal][1];
        }
        else
        {
            minimum_atteint=1;
        }
        //
        //
        // Fin Tant que

    }

    //affichage du resultat
    printf("f(%f,%f)=%f\n",x,y,fonction_inconnue(x,y));
}

//trouve le minimum d'un vecteur
int trouve_min(float V[],int taille)
{
    assert(taille>=1);

    // Initialiser minimum au premier element du vecteur
    // Initialiser indice k_optimal a 0
    //
    // Pour tout les elements d'indice [k] restants du vecteur V
    //  Si minimum > V[k]
    //   Alors affecter minimum=V[k],
    //         et sauvegarder k_optimal=k
    //  Fin Si
    // Fin Pour
    //
    // Retourner k_optimal

    float minimum=V[0];
    int k_optimal=0;

    int k=0;
    for(k=1;k<taille;++k)
    {
        if(minimum>V[k])
        {
            minimum=V[k];
            k_optimal=k;
        }
    }

    return k_optimal;

}

float fonction_inconnue(float x,float y)
{
  return exp(-cos(x-y)+x*x+y*y)+(0.5-sqrt(x*y))*exp(-x*x-0.5*y*y);
}
